import webpack = require('webpack');
import WebpackChain = require('webpack-chain');
import WebpackDevServer = require('webpack-dev-server');
import { AggregatedResult } from '@jest/test-result';
import { GlobalConfig } from '@jest/types/build/Config';
import { Logger } from 'npmlog';
import { IHash, Json, JsonValue, MaybeArray, MaybePromise, JsonArray } from '../types';
declare const PLUGIN_CONTEXT_KEY: ("command" | "commandArgs" | "rootDir" | "userConfig" | "pkg" | "webpack")[];
declare const VALIDATION_MAP: {
    string: "isString";
    number: "isNumber";
    array: "isArray";
    object: "isObject";
    boolean: "isBoolean";
};
export declare type PluginContext = Pick<Context, typeof PLUGIN_CONTEXT_KEY[number]>;
export declare type UserConfigContext = PluginContext & {
    taskName: string;
};
export declare type ValidationKey = keyof typeof VALIDATION_MAP;
export declare type JestResult = {
    results: AggregatedResult;
    globalConfig: GlobalConfig;
};
export interface IOnHookCallbackArg {
    err?: Error;
    args?: CommandArgs;
    stats?: webpack.compilation.MultiStats;
    url?: string;
    devServer?: WebpackDevServer;
    config?: any;
    result?: JestResult;
}
export interface IOnHookCallback {
    (arg?: IOnHookCallbackArg): MaybePromise<void>;
}
export interface IOnHook {
    (eventName: string, callback: IOnHookCallback): void;
}
export interface IPluginConfigWebpack {
    (config: WebpackChain): void;
}
export interface IUserConfigWebpack {
    (config: WebpackChain, value: JsonValue, context: UserConfigContext): Promise<void>;
}
export interface IUserConfigArgs {
    name: string;
    configWebpack?: IUserConfigWebpack;
    defaultValue?: any;
    validation?: ValidationKey;
}
export interface ICliOptionArgs {
    name: string;
    configWebpack?: IUserConfigWebpack;
    commands?: string[];
}
export interface IOnGetWebpackConfig {
    (name: string, fn: IPluginConfigWebpack): void;
    (fn: IPluginConfigWebpack): void;
}
export interface IOnGetJestConfig {
    (fn: IJestConfigFunction): void;
}
export interface IRegisterTask {
    (name: string, chainConfig: WebpackChain): void;
}
export interface IMethodFunction {
    (args?: any): void;
}
export interface IRegsiterMethod {
    (name: string, fn: IMethodFunction): void;
}
export interface IApplyMethod {
    (name: string, ...args: any[]): any;
}
export interface IModifyConfig {
    (userConfig: IUserConfig): IHash<any>;
}
export interface IModifyUserConfig {
    (configKey: string | IModifyConfig, value?: any): void;
}
export interface IPluginAPI {
    log: Logger;
    context: PluginContext;
    registerTask: IRegisterTask;
    getAllTask: () => string[];
    onGetWebpackConfig: IOnGetWebpackConfig;
    onGetJestConfig: IOnGetJestConfig;
    onHook: IOnHook;
    setValue: (name: string, value: any) => void;
    getValue: (name: string) => any;
    registerUserConfig: (args: MaybeArray<IUserConfigArgs>) => void;
    registerCliOption: (args: MaybeArray<ICliOptionArgs>) => void;
    registerMethod: IRegsiterMethod;
    applyMethod: IApplyMethod;
    modifyUserConfig: IModifyUserConfig;
}
export declare type IPluginOptions = Json | JsonArray;
export interface IPlugin {
    (api: IPluginAPI, options?: IPluginOptions): MaybePromise<void>;
}
export declare type CommandName = 'start' | 'build' | 'test';
export declare type CommandArgs = IHash<any>;
export declare type ContextOptions = {
    command: CommandName;
    rootDir: string;
    args: CommandArgs;
};
export declare type TaskConfig = {
    name: string;
    chainConfig: WebpackChain;
    modifyFunctions: Array<IPluginConfigWebpack>;
};
export interface IUserConfig extends Json {
    plugins: Array<string | [string, Json]>;
}
export interface IJestConfigFunction {
    (JestConfig: Json): Json;
}
export declare type IOnGetWebpackConfigArgs = [string, IPluginConfigWebpack] | [IPluginConfigWebpack];
declare class Context {
    command: CommandName;
    commandArgs: CommandArgs;
    rootDir: string;
    webpack: (options: webpack.Configuration[]) => webpack.MultiCompiler;
    configArr: TaskConfig[];
    modifyConfigFns: IOnGetWebpackConfigArgs[];
    modifyJestConfig: Array<IJestConfigFunction>;
    eventHooks: {
        [name: string]: Array<IOnHookCallback>;
    };
    internalValue: IHash<any>;
    userConfigRegistration: IHash<any>;
    cliOptionRegistration: IHash<any>;
    methodRegistration: IHash<any>;
    pkg: Json;
    userConfig: IUserConfig;
    plugins: Array<{
        fn: IPlugin;
        name: string;
        options: IPluginOptions;
    }>;
    constructor({ command, rootDir, args }: ContextOptions);
    _registerConfig: (type: string, args: IUserConfigArgs | ICliOptionArgs | IUserConfigArgs[] | ICliOptionArgs[], parseName?: (name: string) => string) => void;
    _runConfigWebpack(fn: IUserConfigWebpack, configValue: JsonValue): Promise<void>;
    getProjectFile: (fileName: string) => {};
    getUserConfig: () => IUserConfig;
    getPlugins: () => {
        name: string;
        fn: any;
        options: IPluginOptions;
    }[];
    registerTask: IRegisterTask;
    registerMethod: IRegsiterMethod;
    applyMethod: IApplyMethod;
    modifyUserConfig: IModifyUserConfig;
    getAllTask: () => string[];
    onGetWebpackConfig: IOnGetWebpackConfig;
    onGetJestConfig: IOnGetJestConfig;
    runJestConfig: (jestConfig: Json) => Json;
    onHook: IOnHook;
    applyHook: (key: string, opts?: {}) => Promise<void>;
    setValue: (key: string | number, value: any) => void;
    getValue: (key: string | number) => any;
    registerUserConfig: (args: MaybeArray<IUserConfigArgs>) => void;
    registerCliOption: (args: MaybeArray<ICliOptionArgs>) => void;
    runPlugins: () => Promise<void>;
    checkPluginValue: () => void;
    runUserConfig: () => Promise<void>;
    runCliOption: () => Promise<void>;
    runWebpackFunctions: () => Promise<void>;
    setUp: () => Promise<TaskConfig[]>;
}
export default Context;
